@@ -344,7 +344,11 @@ def descriptor(cls, *, size: Optional[int] = None,
         the byte-order of the binary record
     :param bitorder:
         the bit-order of the binary record
-        (must be ``None`` if the *base units* are bytes)
+        (must be ``None`` if the *base units* are bytes).
+        If set to none in bit-based records it is assumed
+        :data:`bpack.enums.EBitOrder.DEFAULT` which corresponds to
+        :data:`bpack.enums.EBitOrder.MSB` in all decoders currently
+        implemented.
     :param baseunits:
         the base units (:data:`bpack.enums.EBaseUnits.BITS` or
         :data:`bpack.enums.EBaseUnits.BYTES`) used to specify the
@@ -493,15 +497,17 @@ def calcsize(obj, units: Optional[EBaseUnits] = None) -> int:
         raise TypeError(f'{obj!r} is not a descriptor')
 
     size = getattr(obj, SIZE_ATTR_NAME)
-    baseunits_ = getattr(obj, BASEUNITS_ATTR_NAME)
 
-    if units and units is not baseunits_:
-        if units is EBaseUnits.BYTES:
-            # baseunits is BITS and units is BYTES
-            size = math.ceil(size / 8)
-        else:
-            # baseunits is BYTES and units is BITS
-            size *= 8
+    if units:
+        baseunits_ = getattr(obj, BASEUNITS_ATTR_NAME)
+        units = EBaseUnits(units)
+        if units is not baseunits_:
+            if units is EBaseUnits.BYTES:
+                # baseunits is BITS and units is BYTES
+                size = math.ceil(size / 8)
+            else:
+                # baseunits is BYTES and units is BITS
+                size *= 8
 
     return size
 
