@@ -9,7 +9,7 @@ import bpack.utils
 import bpack.codecs
 
 from .enums import EBaseUnits
-from .codecs import get_sequence_groups, has_codec, get_codec
+from .codecs import has_codec, get_codec
 from .descriptors import field_descriptors
 
 
@@ -53,6 +53,30 @@ _DEFAULT_SIZE = {
 }
 
 
+def _format_string_without_order(fmt: str, order: str) -> str:
+    # NOTE: in the current implementation the byte order is handled
+    #       externally to _to_fmt
+    # if order != '':
+    #     fmt = fmt[1:] if fmt.startswith(order) else fmt
+
+    # TODO: improve.
+    #       This is mainly a hack necessary because, in the current
+    #       implementation, _to_fmt is always called by Decoder.__init__
+    #       with order='' so here it is not possible to rely on the value
+    #       of order
+    if fmt[0] in {'>', '<', '=', '@', '!'}:
+        if order and fmt[0] != order:
+            raise ValueError(
+                f'inconsistent byteorder for nested record: '
+                f'record byteorder is "{order}", '
+                f'nested record byteorder is "{fmt[0]}"')
+        fmt = fmt[1:]
+    # else:
+    #     # TODO: how to check consistency when order=''?
+
+    return fmt
+
+
 def _to_fmt(type_, size: Optional[int] = None, order: str = '',
             signed: Optional[bool] = None,
             repeat: Optional[int] = None) -> str:
@@ -65,11 +89,11 @@ def _to_fmt(type_, size: Optional[int] = None, order: str = '',
     if has_codec(type_, bpack.codecs.Decoder):
         decoder_ = get_codec(type_)
         if isinstance(decoder_, Decoder):
-            return decoder_.format
+            return _format_string_without_order(decoder_.format, order)
     elif (bpack.is_descriptor(type_) and
           bpack.baseunits(type_) is Decoder.baseunits):
         decoder_ = Decoder(type_)
-        return decoder_.format
+        return _format_string_without_order(decoder_.format, order)
 
     etype = bpack.utils.effective_type(type_)
     repeat = 1 if repeat is None else repeat
@@ -97,7 +121,7 @@ def _enum_converter_factory(type_, converters=None):
         return type_
 
 
-class Decoder(bpack.codecs.Decoder):
+class Decoder(bpack.codecs.BaseStructDecoder):
     """Struct based data decoder.
 
     Default byte-order: MSB.
@@ -110,10 +134,6 @@ class Decoder(bpack.codecs.Decoder):
 
         The *descriptor* parameter* is a bpack record descriptor.
         """
-        super().__init__(descriptor)
-
-        assert bpack.bitorder(descriptor) is None
-
         byteorder = bpack.byteorder(descriptor)
         # assert all(descr.order for descr in field_descriptors(descriptor))
         byteorder = byteorder.value
@@ -137,33 +157,14 @@ class Decoder(bpack.codecs.Decoder):
             if bpack.utils.is_enum_type(field_descr.type)
         )
 
-        self._codec = struct.Struct(fmt)
-        self._converters = [
+        codec = struct.Struct(fmt)
+        converters = [
             (idx, converters_map[field_descr.type])
-            for idx, field_descr in enumerate(
-                field_descriptors(self.descriptor))
+            for idx, field_descr in enumerate(field_descriptors(descriptor))
             if field_descr.type in converters_map
         ]
-        self._groups = get_sequence_groups(descriptor)
-
-    @property
-    def format(self) -> str:
-        """Return the :mod:`struct` format string."""
-        return self._codec.format
-
-    def decode(self, data: bytes):
-        """Decode binary data and return a record object."""
-        values = list(self._codec.unpack(data))
-
-        for idx, func in self._converters:
-            values[idx] = func(values[idx])
-
-        for type_, slice_ in self._groups[::-1]:
-            subtype = type_(values[slice_])
-            del values[slice_]
-            values.insert(slice_.start, subtype)
-
-        return self.descriptor(*values)
+        super().__init__(descriptor, codec, converters)
+        assert bpack.bitorder(descriptor) is None
 
 
 decoder = bpack.codecs.make_codec_decorator(Decoder)
