@@ -2,6 +2,7 @@
 
 import sys
 import enum
+import struct
 
 from typing import List, Sequence
 
@@ -713,22 +714,25 @@ class TestNestedRecord:
     @staticmethod
     def get_encoded_data(baseunits):
         if baseunits is bpack.EBaseUnits.BYTES:
+            # TODO: use the default byte order
             encoded_data = bytes([
                 0b00000000, 0b00000000, 0b00000000, 0b00000000,
                 0b00000001, 0b00000000, 0b00000000, 0b00000000,
                 0b00000010, 0b00000000, 0b00000000, 0b00000000,
                 0b00000011, 0b00000000, 0b00000000, 0b00000000,
+                0b00000001, 0b00000000, 0b00000000, 0b00000000,
+                0b00000010, 0b00000000, 0b00000000, 0b00000000,
             ])
 
         else:  # baseunits is bpack.EBaseUnits.BITS:
-            encoded_data = bytes([0b00000001, 0b00100011])
+            encoded_data = bytes([0b00000001, 0b00100011, 0b000010010])
 
         return encoded_data
 
     def test_nested_record_decoders(self, backend):
         encoded_data = self.get_encoded_data(backend.Decoder.baseunits)
 
-        @backend.decoder
+        @backend.decoder  # NOTE: this is a decoder
         @bpack.descriptor(baseunits=backend.Decoder.baseunits)
         class Record:
             field_1: int = bpack.field(size=4, default=1)
@@ -740,12 +744,14 @@ class TestNestedRecord:
             field_1: int = bpack.field(size=4, default=0)
             field_2: Record = Record()
             field_3: int = bpack.field(size=4, default=3)
+            field_4: Record = Record()
 
         assert NestedRecord.frombytes(encoded_data) == NestedRecord()
 
     def test_nested_record(self, backend):
         encoded_data = self.get_encoded_data(backend.Decoder.baseunits)
 
+        # NOTE: this time the inner record is not a decoder
         @bpack.descriptor(baseunits=backend.Decoder.baseunits)
         class Record:
             field_1: int = bpack.field(size=4, default=1)
@@ -757,5 +763,169 @@ class TestNestedRecord:
             field_1: int = bpack.field(size=4, default=0)
             field_2: Record = Record()
             field_3: int = bpack.field(size=4, default=3)
+            field_4: Record = Record()
 
         assert NestedRecord.frombytes(encoded_data) == NestedRecord()
+
+    def test_nested_record_decoders_with_order(self, backend):
+        encoded_data = self.get_encoded_data(backend.Decoder.baseunits)
+        if backend.Decoder.baseunits is bpack.EBaseUnits.BITS:
+            kwargs = dict(bitorder='>', byteorder='>')
+        else:
+            # TODO: use the default byteorder (see get_encoded_data)
+            kwargs = dict(byteorder=bpack.EByteOrder.LE)
+
+        @backend.decoder  # NOTE: this is a decoder
+        @bpack.descriptor(baseunits=backend.Decoder.baseunits, **kwargs)
+        class Record:
+            field_1: int = bpack.field(size=4, default=1)
+            field_2: int = bpack.field(size=4, default=2)
+
+        @backend.decoder
+        @bpack.descriptor(baseunits=backend.Decoder.baseunits, **kwargs)
+        class NestedRecord:
+            field_1: int = bpack.field(size=4, default=0)
+            field_2: Record = Record()
+            field_3: int = bpack.field(size=4, default=3)
+            field_4: Record = Record()
+
+        assert NestedRecord.frombytes(encoded_data) == NestedRecord()
+
+
+@pytest.mark.parametrize(
+    'backend',
+    [pytest.param(bpack.st, id='st'),
+     # pytest.param(bpack_np, id='np',
+     #              marks=skipif(not bpack_np, reason='not available'))
+     pytest.param(bpack_bs, id='bs',
+                  marks=skipif(not bpack_bs, reason='not available'))])
+class TestMultiNestedRecord:
+    def _record_to_list(self, record):
+        from bpack.utils import is_sequence_type
+        out = []
+        for filed in bpack.fields(record):
+            value = getattr(record, filed.name)
+            if bpack.is_descriptor(value):
+                out.extend(self._record_to_list(value))
+            elif is_sequence_type(filed.type):
+                out.extend(value)
+            else:
+                out.append(value)
+        return out
+
+    def _bytes_record_to_data(self, record):
+        from bpack import st as _st
+        values = self._record_to_list(record)
+        decoder = _st.Decoder(type(record))
+        fmt = decoder.format
+        values = [getattr(v, 'value', v) for v in values]
+        return struct.pack(fmt, *values)
+
+    def _bits_record_to_data(self, record):
+        # TODO: check
+        import bitstruct
+        from bpack import bs as _bs
+        values = self._record_to_list(record)
+        decoder = _bs.Decoder(type(record))
+        fmt = decoder.format
+        values = [getattr(v, 'value', v) for v in values]
+        return bitstruct.pack(fmt, *values)
+
+    def _record_to_data(self, record):
+        baseunits = bpack.baseunits(record)
+        if baseunits is bpack.EBaseUnits.BYTES:
+            data = self._bytes_record_to_data(record)
+        else:
+            data = self._bits_record_to_data(record)
+        nbytes = bpack.calcsize(record, bpack.EBaseUnits.BYTES)
+        assert len(data) == nbytes
+        return data
+
+    def test_nested_record_two_levels(self, backend):
+        class EEnum(enum.Enum):
+            ONE = 1
+            FOUR = 4
+
+        @bpack.descriptor(baseunits=backend.Decoder.baseunits)
+        class RecordLevel02:
+            field_01: EEnum = bpack.field(size=4, default=EEnum.ONE)
+            field_02: int = bpack.field(size=4, default=2)
+
+        @backend.decoder
+        @bpack.descriptor(baseunits=backend.Decoder.baseunits)
+        class NestedRecord:
+            field_1: int = bpack.field(size=4, default=0)
+            field_2: RecordLevel02 = RecordLevel02()
+            field_3: int = bpack.field(size=4, default=3)
+            field_4: RecordLevel02 = RecordLevel02(EEnum.FOUR, 5)
+
+        record = NestedRecord()
+        encoded_data = self._record_to_data(record)
+        assert NestedRecord.frombytes(encoded_data) == record
+
+    def test_nested_record_three_levels(self, backend):
+        class EEnum(enum.Enum):
+            ONE = 1
+            TWO = 2
+            SEVEN = 7
+
+        @bpack.descriptor(baseunits=backend.Decoder.baseunits)
+        class RecordLevel03:
+            field_001: EEnum = bpack.field(size=4, default=EEnum.ONE)
+            field_002: int = bpack.field(size=4, default=2)
+
+        @bpack.descriptor(baseunits=backend.Decoder.baseunits)
+        class RecordLevel02:
+            field_01: int = bpack.field(size=4, default=1)
+            field_02: RecordLevel03 = RecordLevel03(EEnum.TWO, 3)
+            field_03: int = bpack.field(size=4, default=4)
+
+        @backend.decoder
+        @bpack.descriptor(baseunits=backend.Decoder.baseunits)
+        class NestedRecord:
+            field_1: int = bpack.field(size=4, default=0)
+            field_2: RecordLevel02 = RecordLevel02()
+            field_3: int = bpack.field(size=4, default=5)
+            field_4: RecordLevel02 = RecordLevel02(
+                6, RecordLevel03(EEnum.SEVEN, 8), 9)
+
+        record = NestedRecord()
+        encoded_data = self._record_to_data(record)
+        assert NestedRecord.frombytes(encoded_data) == record
+
+    def test_nested_record_four_levels(self, backend):
+        class EEnum(enum.Enum):
+            ONE = 1
+            THREE = 3
+            TEN = 10
+
+        @bpack.descriptor(baseunits=backend.Decoder.baseunits)
+        class RecordLevel04:
+            field_0001: EEnum = bpack.field(size=4, default=EEnum.ONE)
+            field_0002: int = bpack.field(size=4, default=2)
+
+        @bpack.descriptor(baseunits=backend.Decoder.baseunits)
+        class RecordLevel03:
+            field_001: int = bpack.field(size=4, default=1)
+            field_002: RecordLevel04 = RecordLevel04()
+            field_003: int = bpack.field(size=4, default=3)
+
+        @bpack.descriptor(baseunits=backend.Decoder.baseunits)
+        class RecordLevel02:
+            field_01: int = bpack.field(size=4, default=1)
+            field_02: RecordLevel03 = RecordLevel03(
+                2, RecordLevel04(EEnum.THREE, 4), 5)
+            field_03: int = bpack.field(size=4, default=6)
+
+        @backend.decoder
+        @bpack.descriptor(baseunits=backend.Decoder.baseunits)
+        class NestedRecord:
+            field_1: int = bpack.field(size=4, default=0)
+            field_2: RecordLevel02 = RecordLevel02()
+            field_3: int = bpack.field(size=4, default=7)
+            field_4: RecordLevel02 = RecordLevel02(
+                8, RecordLevel03(9, RecordLevel04(EEnum.TEN, 11), 12), 13)
+
+        record = NestedRecord()
+        encoded_data = self._record_to_data(record)
+        assert NestedRecord.frombytes(encoded_data) == record
