@@ -1,7 +1,7 @@
 """Test bpack field descriptors."""
 
+import sys
 import enum
-import dataclasses
 
 from typing import List
 
@@ -16,22 +16,12 @@ class TestFieldFactory:
     def test_base():
         bpack.field(size=1, offset=0, signed=False, default=0, repeat=1)
 
-    @staticmethod
-    def test_empty_field():
-        with pytest.raises(TypeError):
-            bpack.field()
-
     @staticmethod
     def test_field_vs_field_class():
         field_ = bpack.field(size=1)
         assert bpack.descriptors.is_field(field_)
         assert isinstance(field_, bpack.descriptors.Field)
 
-    @staticmethod
-    def test_missing_size():
-        with pytest.raises(TypeError):
-            bpack.field(default=1/3)
-
     @staticmethod
     @pytest.mark.parametrize(argnames='size', argvalues=[1.3, 'x'])
     def test_invalid_size_type(size):
@@ -59,7 +49,7 @@ class TestFieldFactory:
     @pytest.mark.parametrize('value', [-8, 'a'])
     def test_invalid_signed_type(value):
         with pytest.raises(TypeError):
-            bpack.field(size=8, default=1, signed=value)
+            bpack.field(size=8, default=1, signed=value)                # noqa
 
     @staticmethod
     @pytest.mark.parametrize(argnames='repeat', argvalues=[1.3, 'x'])
@@ -82,7 +72,6 @@ class TestRecordFields:
     @staticmethod
     def test_field_properties_01():
         @bpack.descriptor
-        @dataclasses.dataclass
         class Record:
             field_1: int = bpack.field(size=4, default=0, signed=True)
             field_2: float = bpack.field(size=8, default=1/3)
@@ -109,7 +98,6 @@ class TestRecordFields:
     @staticmethod
     def test_field_properties_02():
         @bpack.descriptor
-        @dataclasses.dataclass
         class Record:
             field_1: int = bpack.field(size=4, offset=1, default=0,
                                        signed=False)
@@ -137,7 +125,6 @@ class TestRecordFields:
     @staticmethod
     def test_field_properties_03():
         @bpack.descriptor
-        @dataclasses.dataclass
         class Record:
             field_1: int = bpack.field(size=4, offset=1, default=0)
             field_2: float = bpack.field(size=8, offset=6, default=1/3)
@@ -159,6 +146,13 @@ class TestRecordFields:
             assert field_descr.signed == signed
             assert field_descr.repeat == repeat
 
+    @staticmethod
+    def test_finvalid_field_type():
+        with pytest.raises(TypeError):
+            @bpack.descriptor
+            class Record:                                       # noqa
+                field_1: 'invalid' = bpack.field(size=4)        # noqa: F821
+
 
 class TestEnumFields:
     @staticmethod
@@ -169,7 +163,6 @@ class TestEnumFields:
             C = 'c'
 
         @bpack.descriptor
-        @dataclasses.dataclass
         class Record:
             field_1: int = bpack.field(size=4, default=0)
             field_2: EEnumType = bpack.field(size=1, default=EEnumType.A)
@@ -188,7 +181,6 @@ class TestEnumFields:
             C = 4
 
         @bpack.descriptor
-        @dataclasses.dataclass
         class Record:
             field_1: int = bpack.field(size=4, default=0)
             field_2: EEnumType = bpack.field(size=1, signed=True,
@@ -208,7 +200,6 @@ class TestEnumFields:
             C = 4
 
         @bpack.descriptor
-        @dataclasses.dataclass
         class Record:
             field_1: int = bpack.field(size=4, default=0)
             field_2: EEnumType = bpack.field(size=1, default=EEnumType.A)
@@ -228,8 +219,7 @@ class TestEnumFields:
 
         with pytest.raises(TypeError):
             @bpack.descriptor
-            @dataclasses.dataclass
-            class Record:
+            class Record:                                               # noqa
                 field_1: int = bpack.field(size=4, default=0)
                 field_2: EEnumType = bpack.field(size=1, default=EEnumType.A)
 
@@ -242,8 +232,7 @@ class TestEnumFields:
 
         with pytest.warns(UserWarning):
             @bpack.descriptor
-            @dataclasses.dataclass
-            class Record:
+            class Record:                                               # noqa
                 field_1: int = bpack.field(size=4, default=0)
                 field_2: EEnumType = bpack.field(size=1, signed=True,
                                                  default=EEnumType.A)
@@ -285,16 +274,17 @@ class TestFieldDescriptor:
     @staticmethod
     def test_init_invalid_type():
         with pytest.raises(TypeError):
-            bpack.descriptors.BinFieldDescriptor(size=1.1)
+            bpack.descriptors.BinFieldDescriptor(size=1.1)              # noqa
 
         with pytest.raises(TypeError):
-            bpack.descriptors.BinFieldDescriptor(offset=2.1)
+            bpack.descriptors.BinFieldDescriptor(offset=2.1)            # noqa
 
         with pytest.raises(TypeError):
-            bpack.descriptors.BinFieldDescriptor(signed=complex(3.1, 0))
+            bpack.descriptors.BinFieldDescriptor(
+                signed=complex(3.1, 0))                                 # noqa
 
         with pytest.raises(TypeError):
-            bpack.descriptors.BinFieldDescriptor(repeat=1.1)
+            bpack.descriptors.BinFieldDescriptor(repeat=1.1)            # noqa
 
     @staticmethod
     def test_init_invalid_value():
@@ -458,3 +448,159 @@ class TestFieldDescriptor:
         assert descr.is_int_type()
         assert not descr.is_sequence_type()
         assert descr.is_enum_type()
+
+
+class TestAnnotatedType:
+    @staticmethod
+    @pytest.mark.parametrize('byteorder', ['>', '<', '|', ''],
+                             ids=['>', '<', '|', 'None'])
+    def test_annotated_type(byteorder):
+        @bpack.descriptor(byteorder=byteorder if byteorder != '|' else '')
+        class Record:
+            field_1: bpack.T[f'{byteorder}i4']                  # noqa: F821
+            field_2: bpack.T[f'{byteorder}u4']                  # noqa: F821
+            field_3: bpack.T[f'{byteorder}f4']                  # noqa: F821
+            field_4: bpack.T[f'{byteorder}c4']                  # noqa: F821
+            field_5: bpack.T[f'{byteorder}S4']                  # noqa: F821
+
+        fields = dict(
+            (field.name, get_field_descriptor(field))
+            for field in bpack.fields(Record)
+        )
+
+        assert fields['field_1'].type == int
+        assert fields['field_1'].size == 4
+        assert fields['field_1'].signed is True
+        assert fields['field_1'].repeat is None
+
+        assert fields['field_2'].type == int
+        assert fields['field_2'].size == 4
+        assert fields['field_2'].signed is False
+        assert fields['field_2'].repeat is None
+
+        assert fields['field_3'].type == float
+        assert fields['field_3'].size == 4
+        assert fields['field_3'].signed is None
+        assert fields['field_3'].repeat is None
+
+        assert fields['field_4'].type == complex
+        assert fields['field_4'].size == 4
+        assert fields['field_4'].signed is None
+        assert fields['field_4'].repeat is None
+
+        assert fields['field_5'].type == bytes
+        assert fields['field_5'].size == 4
+        assert fields['field_5'].signed is None
+        assert fields['field_5'].repeat is None
+
+    @staticmethod
+    def test_list_with_annotated_type():
+        typestr = 'i4'
+
+        @bpack.descriptor
+        class Record:
+            field_1: List[bpack.T[typestr]] = bpack.field(repeat=2)
+
+        field = bpack.fields(Record)[0]
+        assert field.type == List[bpack.T[typestr]]
+
+        field_descr = get_field_descriptor(field)
+        assert field_descr.type == List[int]
+        assert field_descr.size == 4
+        assert field_descr.signed is True
+        assert field_descr.repeat == 2
+
+    @staticmethod
+    def test_byteorder_consistency():
+        typestr = '>i8'
+        with pytest.raises(bpack.descriptors.DescriptorConsistencyError):
+            @bpack.descriptor(byteorder=bpack.EByteOrder.LE)
+            class Record:                                       # noqa
+                field: bpack.T[typestr]
+
+        typestr = '<i8'
+        with pytest.raises(bpack.descriptors.DescriptorConsistencyError):
+            @bpack.descriptor(byteorder=bpack.EByteOrder.BE)   # noqa: F811
+            class Record:                                       # noqa
+                field: bpack.T[typestr]
+
+        typestr = '>i8' if sys.byteorder == 'little' else '<i8'
+        with pytest.raises(bpack.descriptors.DescriptorConsistencyError):
+            @bpack.descriptor                                   # noqa: F811
+            class Record:                                       # noqa
+                field: bpack.T[typestr]
+
+        typestr = '<i8' if sys.byteorder == 'little' else '>i8'
+
+        @bpack.descriptor                                       # noqa: F811
+        class Record:                                           # noqa
+            field: bpack.T[typestr]
+
+        typestr = '|i8'
+
+        @bpack.descriptor(byteorder=bpack.EByteOrder.BE)       # noqa: F811
+        class Record:                                           # noqa
+            field: bpack.T[typestr]
+
+    @staticmethod
+    def test_size_consistency():
+        typestr = 'i8'
+
+        @bpack.descriptor
+        class Record:
+            field: bpack.T[typestr] = bpack.field(size=8, signed=True)
+
+        descr = get_field_descriptor(bpack.fields(Record)[0])
+        assert descr.type is int
+        assert descr.size == 8
+        assert descr.repeat is None
+        assert descr.signed is True
+
+        typestr = 'u8'
+
+        @bpack.descriptor
+        class Record:
+            field: bpack.T[typestr] = bpack.field(size=8, signed=False)
+
+        descr = get_field_descriptor(bpack.fields(Record)[0])
+        assert descr.type is int
+        assert descr.size == 8
+        assert descr.repeat is None
+        assert descr.signed is False
+
+        typestr = 'f8'
+
+        @bpack.descriptor
+        class Record:
+            field: bpack.T[typestr] = bpack.field(size=8)
+
+        descr = get_field_descriptor(bpack.fields(Record)[0])
+        assert descr.type is float
+        assert descr.size == 8
+        assert descr.repeat is None
+        assert descr.signed is None
+
+    @staticmethod
+    def test_size_consistency_error():
+        typestr = 'i8'
+        with pytest.raises(bpack.descriptors.DescriptorConsistencyError):
+            @bpack.descriptor
+            class Record:                                               # noqa
+                field: bpack.T[typestr] = bpack.field(size=3)
+
+    @staticmethod
+    def test_signed_consistency_error():
+        typestr = 'i8'
+        with pytest.raises(bpack.descriptors.DescriptorConsistencyError):
+            @bpack.descriptor
+            class Record:                                               # noqa
+                field: bpack.T[typestr] = bpack.field(size=8, signed=False)
+
+    @staticmethod
+    @pytest.mark.parametrize('typestr',
+                             ['invalid', '@i8', '|x8', '|i0', '|ii'])
+    def test_invalid_typestr(typestr):
+        with pytest.raises(ValueError):
+            @bpack.descriptor
+            class Record:                                               # noqa
+                field: bpack.T[typestr] = bpack.field(size=1)
